/*-----------------------------------------------------------------------------
 * DISCLAIMER.
 * This is a script for demonstrating the SELECT... FOR XML... solution
 * for generating Nested XML from a data base. This script is intended to
 * demonstrate some programming principles and no warranty is either 
 * expressed or implied.
 *
 *                   RUN THIS SCRIPT AT YOUR OWN RISK
 * If you have any questions or comments, please contact me at:
 * http://www.tomasvera.com
 *---------------------------------------------------------------------------*/
 

/*-----------------------------------------------------------------------------
 * These are some sample queries used to demonstrate creating Nested XML
 * from Transact SQL. It is assumed that you have already run the 
 * "NestedXML.sql" script located in this ZIP/bundle.
 *----------------------------------------------------------------------------*/

-- The common way to get all the information in a single join.
-- To get all the data you need in one pass, join the tables.
-- returns a large amount of data
Select c.*, o.*, i.*, n.*
FROM customers c join orders o on o.CustomerId = c.CustomerID
JOIN OrderItems i on i.OrderId = o.OrderId
JOIN CustNotes n on n.CustomerID = c.CustomerId
--------------------------------------------------------------------------------

-- Now, let's use the the 'FOR XML' clause.

-- Do a normal nested select query with just the order data
-- results look promising...
Select customers.*, orders.*, items.*
FROM customers join orders on orders.CustomerId = customers.CustomerID
JOIN OrderItems items on items.OrderId = orders.OrderId
FOR XML AUTO, TYPE
-- Note: there are multiple 'customers' and no outer Node to contain
-- the customer nodes. If you do not provide a 'wrapper' this XML
-- will be invalid
--------------------------------------------------------------------------------


-- Now, add in the CustNotes information and try the same
-- query again. 
Select customers.*, orders.*, items.*, notes.*
FROM customers join orders on orders.CustomerId = customers.CustomerID
JOIN OrderItems items on items.OrderId = orders.OrderId
JOIN CustNotes notes on notes.CustomerID = customers.CustomerId
FOR XML AUTO, TYPE
-- Note that now we have 'Notes' under 'Items'
-- Not what we wanted. Also the previous note about no outer Node
-- is still valid here as well.
--------------------------------------------------------------------------------

--
-- Now do a XML Explicit query. More work to set up, but
-- gives the results expected.
-- [DISCLAIMER: This is my first experience with XML EXPLICIT
-- If I have made an error, please contact me and provide the
-- proper syntax so that I can update this tutorial! ]
--
select 1 as TAG, NULL as PARENT, 'DOC' as [doc!1!doc]
	, NULL as [Customer!2!CustomerId], NULL as [Customer!2!LastName]
	, NULL as [Orders!3!CustomerId]
	, null as [Order!4!OrderId], null as [Order!4!CustomerId],null as [Order!4!Total]
	, NULL as[Items!5!OrderId]
	, NULL as [Item!6!LineItemId],null as [Item!6!OrderId], null as [Item!6!SKU], null as [Item!6!ExtPrice]
	, NULL as [Notes!7!CustomerId]
	, NULL as [Notes!8!NoteId], NULL as [Notes!8!NoteText]
UNION ALL
SELECT 2 as TAG, 1 as PARENT, 'DOC' as [doc!1!doc]
	, CustomerId as [Customer!2!CustomerId], LastName as [Customer!2!LastName]
	, NULL as [Order!3!CustomerId]
	, null as [Order!4!OrderId],null as [Order!4!CustomerId], null as [Order!4!Total]
	, NULL as[Items!5!OrderId]
	, NULL as [Item!6!LineItemId],null as [Item!6!OrderId], null as [Item!6!SKU], null as [Item!6!ExtPrice]
	, NULL as [Notes!7!CustomerId]
	, NULL as [Notes!8!NoteId], NULL as [Notes!8!NoteText]
	FROM Customers
UNION ALL
SELECT 3 as TAG, 2 as PARENT, 'DOC' as [doc!1!doc]
	, CustomerId as [Customer!2!CustomerId], LastName as [Customer!2!LastName]
	, CustomerId as [Order!3!CustomerId]
	, null as [Order!4!OrderId],null as [Order!4!CustomerId], null as [Order!4!Total]
	, NULL as[Items!5!OrderId]
	, NULL as [Item!6!LineItemId],null as [Item!6!OrderId], null as [Item!6!SKU], null as [Item!6!ExtPrice]
	, NULL as [Notes!7!CustomerId]
	, NULL as [Notes!8!NoteId], NULL as [Notes!8!NoteText]
	FROM Customers
UNION ALL
SELECT 4 as TAG, 3 as PARENT, 	'DOC' as [doc!1!doc]
	, CustomerId as [Customer!2!CustomerId], NULL as [Customer!2!LastName]
	, CustomerId as [Order!3!CustomerId]
	, OrderID as [Order!4!OrderId],CustomerID as [Order!4!CustomerId],  Total as [Order!4!Total]
	, NULL as[Items!5!OrderId]
	, NULL as [Item!6!LineItemId],null as [Item!6!OrderId], null as [Item!6!SKU], null as [Item!6!ExtPrice]
	, NULL as [Notes!7!CustomerId]
	, NULL as [Notes!8!NoteId], NULL as [Notes!8!NoteText]
	FROM Orders
UNION ALL
SELECT 5 as TAG, 4 as PARENT, 	'DOC' as [doc!1!doc]
	, CustomerId as [Customer!2!CustomerId], NULL as [Customer!2!LastName]
	, CustomerId as [Order!3!CustomerId]
	, OrderID as [Order!4!OrderId],CustomerID as [Order!4!CustomerId],  Total as [Order!4!Total]
	, OrderID as [Items!5!OrderId]
	, NULL as [Item!6!LineItemId],null as [Item!6!OrderId], null as [Item!6!SKU], null as [Item!6!ExtPrice]
	, NULL as [Notes!7!CustomerId]
	, NULL as [Notes!8!NoteId], NULL as [Notes!8!NoteText]
	FROM Orders
UNION ALL
SELECT 6 as TAG, 5 as PARENT , 'DOC' as [doc!1!doc]
	, o.CustomerID as [Customer!2!CustomerId], NULL as [Customer!2!LastName]
	, o.CustomerID as [Order!3!CustomerId]
	, oi.OrderId as [Order!4!OrderId], o.CustomerID as [Order!4!CustomerId],  null as [Order!4!Total]
	, oi.OrderId as [Items!5!OrderId]
	, LineItemID as [Item!6!LineItemId], oi.ORderId as [Item!6!OrderId], SKU as [Item!6!SKU], EXTPRICE as [Item!6!ExtPrice]
	, NULL as [Notes!7!CustomerId]
	, NULL as [Notes!8!NoteId], NULL as [Notes!8!NoteText]
	From OrderItems oi join Orders o on oi.OrderID = o.OrderId
UNION ALL
SELECT 7 as TAG, 2 as PARENT, 'DOC' as [doc!1!doc]
	, CustomerId as [Customer!2!CustomerId], LastName as [Customer!2!LastName]
	, NULL as [Order!3!CustomerId]
	, null as [Order!4!OrderId],null as [Order!4!CustomerId], null as [Order!4!Total]
	, NULL as[Items!5!OrderId]
	, NULL as [Item!6!LineItemId],null as [Item!6!OrderId], null as [Item!6!SKU], null as [Item!6!ExtPrice]
	, CustomerId as [Notes!7!CustomerId]
	, NULL as [Notes!8!NoteId], NULL as [Notes!8!NoteText]
	FROM Customers
UNION ALL
SELECT 8 as TAG, 7 as PARENT, 'DOC' as [doc!1!doc]
	, n.CustomerId as [Customer!2!CustomerId], NULL as [Customer!2!LastName]
	, NULL as [Order!3!CustomerId]
	, null as [Order!4!OrderId],null as [Order!4!CustomerId], null as [Order!4!Total]
	, NULL as[Items!5!OrderId]
	, NULL as [Item!6!LineItemId],null as [Item!6!OrderId], null as [Item!6!SKU], null as [Item!6!ExtPrice]
	, n.CustomerId as [Notes!7!CustomerId]
	, CustNoteId as [Notes!8!NoteId], NoteText as [Notes!8!NoteText]
	FROM CustNotes n join Customers c on n.CustomerId=c.CustomerId
ORDER BY 
	[Customer!2!CustomerId],[Orders!3!CustomerId],[Order!4!OrderId],[Items!5!OrderId],[Item!6!LineItemId],
	[Notes!7!CustomerId],[Notes!8!NoteId],
	[doc!1!doc] 
FOR XML EXPLICIT
-- Notice the grouping is as we expect and that the outermost node
-- is present.
-- if you have any questions or comments, please contact me at:
-- http://www.tomasvera.com
--